#	  This file is part of Dim, a Digital Image Management program.
#	  Copyright 2003, Anssi Piirainen
#    Dim/EXIFView.py - Shows EXIF metadata for the specified file
#
#    Dim is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Dim is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Dim; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
import EXIF

class EXIFView:
	
	shownTags = { "EXIF DateTimeDigitized" : "Date/time", \
		"EXIF ExifImageLength" : "Height", \
		"EXIF ExifImageWidth" : "Width", \
		"EXIF ExposureBiasValue" : "Exposure bias", \
		"EXIF ExposureTime" : "Exposure time", \
		"MakerNote ExposureMode" : "Exposure mode", \
		"EXIF Flash" : "Flash used", \
		"MakerNote FlashMode" : "Flash mode", \
		"EXIF ShutterSpeedValue" : "Shutter speed", \
		"MakerNote MeteringMode" : "Metering mode", \
		"MakerNote ISO" : "ISO", \
		"EXIF FNumber" : "F-number (ratio)", \
		"EXIF FocalLength" : "Focal length", \
		"MakerNote FocusType" : "Focus type", \
		"MakerNote FocusMode" : "Focus mode", \
		"MakerNote AFPointSelected" : "AF point", \
		"MakerNote EasyShootingMode" : "Easy shooting mode", \
		"MakerNote Macromode" : "Macro mode", \
		"MakerNote SubjectDistance" : "Subject distance", \
		"MakerNote WhiteBalance" : "White balance setting", \
		"Image Make" : "Camera maker", \
		"Image Model" : "Camera model" }
	
	def __init__(self, imageFile):
		try:
			self.exif = EXIF.process_file(open(imageFile, 'rb'), 0)
		except IOError:
			print "unable to extract EXIF tag from image ", imageFile
			raise
	
	def getExifFormatted(self):
		x = [key for key in self.exif.keys() if key in self.shownTags]
		x.sort()
		result = ""
		for i in x:
			if i in ('JPEGThumbnail', 'TIFFThumbnail'):
				continue
			try:
				if i[:9] == "MakerNote":
					result = result + '   %s: %s ' \
						% (self.shownTags[i], self.exif[i]) + '\n'
				else:
					result = result + '   %s: %s ' \
						% (self.shownTags[i], self.exif[i].printable) + '\n'
			except AttributeError:
				print i, " value = ", self.exif[i]
		return result
 