#	  This file is part of Dim, a Digital Image Management program.
#	  Copyright 2003, Anssi Piirainen
#    Dim/ImageLib.py - Image manipulation routines
#
#    Dim is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Dim is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Dim; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
import os
from stat import *
import md5
import Image
import gtk

def isImage(fileNameWithPath):
	"""Checks whether the given file contains an image."""
	
	try:
		image = Image.open(fileNameWithPath)
	except IOError:
		return False
	print "found image ", fileNameWithPath
	return True

def createThumbnail(filename, thumbnailDir):
	pixbuf = loadAndScalePixbuf(filename, 128, 128)
	thumbFilename = os.path.join(thumbnailDir, generateThumbFilename(filename))
	options = { }
	try:
		options = { "tEXt::Thumb::URI" : "file://" + filename.encode("UTF-8"), "tEXt::Thumb::MTime" : "%s" % getMTime(filename) }
	except UnicodeError:
		print "unicode error while encoding ", filename
		#raise
	pixbuf.save(thumbFilename, "png", options)
	return thumbFilename
	
def generateThumbFilename(originalImageFilenameWithPath):
	"""Generate filename for a thumbnal. The thumbnail is for the image
	passed in the argument.
	
	Returns string"""
	
	m = md5.new()
	m.update("file://" + originalImageFilenameWithPath)
	return m.hexdigest() + ".png"

class ThumbnailFactory:

	def __init__(self, thumbDir):
		self.thumbDir = thumbDir
	
	def getThumbDir(self):
		return self.thumbDir

	def findThumbnail(self, originalImageFilenameWithPath):
		"""Checks if a thumbnail exists for this image file and returns the thumbnail file name (with full path).
		If the thumbnail does not exist, returns None.
		
		Returns string"""
		
		thumbnail = os.path.join(self.thumbDir, generateThumbFilename(originalImageFilenameWithPath))
		return isImage(thumbnail) and thumbnail or None

def loadAndScalePixbuf(filename, to_width, to_height):
	pixbuf = gtk.gdk.pixbuf_new_from_file(filename)

	factor_x = float(to_width)/float(pixbuf.get_width())
	factor_y = float(to_height)/float(pixbuf.get_height())
	factor = factor_x < factor_y and factor_x or factor_y

	print "scaling with factor ", factor
		
	#pixbuf.scale(scaled_pixbuf, 0, 0, pixbuf.get_width(), pixbuf.get_height(), 0, 0, 1, 1, gtk.gdk.INTERP_BILINEAR)
	return pixbuf.scale_simple(int(pixbuf.get_width() * factor), int(pixbuf.get_height() * factor), gtk.gdk.INTERP_BILINEAR)

def getMTime(fileNameWithPath):
	return os.stat(fileNameWithPath)[ST_MTIME]
	