#	  This file is part of Dim, a Digital Image Management program.
#	  Copyright 2003, Anssi Piirainen
#    Dim/DimController.py - GUI initialization and control
#
#    Dim is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Dim is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Dim; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
import gtk
import gtk.glade
import gnome.ui
from gtk.keysyms import *
import gc

import DirTreeModel
from dimlib.Library import *
import ThumbGrid
import GConf

class DimController:
	
	def __init__(self, thumbDir, glade_xml, startDir):
		self.xml = glade_xml
		self.window = glade_xml.get_widget("Dim")

		self.config = GConf.GConf("dim")
		self.__restoreWindowSize(self.config, self.window)

		self.dirTreeView = glade_xml.get_widget("dir_tree")
		
		selection = self.dirTreeView.get_selection()
		selection.set_mode(gtk.SELECTION_SINGLE)
		
		self.dirTreeModel = DirTreeModel.DirTreeModel()
		self.dirTreeView.set_model(self.dirTreeModel.getModel())
		
		renderer = gtk.CellRendererText()
		column = gtk.TreeViewColumn("Folder", renderer, text=0)
		self.dirTreeView.append_column(column)
		
		imageIndex = Index(thumbDir)
		scroller = glade_xml.get_widget("scrolledwindow2")
		scroller.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		
		# the IconList is created here, because we cannot retrieve it from the glade_xml correctly
		thumbGrid = ThumbGrid.ThumbGrid(imageIndex)
		scroller.add(thumbGrid)
		thumbGrid.show()
		self.thumbGrid = thumbGrid

		self.__setRootFolder(startDir)
		glade_xml.get_widget("entry1").set_text(self.rootFolder)
		
		self.imageContainer = glade_xml.get_widget("scrolledwindow4")
		self.imageNotebook = glade_xml.get_widget("notebook1")
		self.previewNotebook = glade_xml.get_widget("notebook2")

		self.dirTreeView.connect("select-cursor-row", self.__onSelectFolder)
		self.dirTreeView.connect("row-expanded", self.__onRowExpanded)
		self.dirTreeView.connect("row-activated", self.__onFolderActivate)
		
		self.window.connect("delete_event", self.__onDeleteEvent)
		self.window.connect("destroy", self.__onDestroy)
		self.window.connect("key_press_event", self.__on_key, glade_xml.get_widget("image1"))

		thumbGrid.connect("button_press_event", self.__onThumbnailClicked)
		thumbGrid.connect("select_icon", self.__onSelectIcon, glade_xml.get_widget("textview1"))

		handlers = {
			"on_thumbnails_activate" : (self.__on_thumbnailsmenuitem_activate),
			"on_images_activate" : (self.__on_imagemenuitem_activate),
			"on_next_image1_activate" : (self.__on_next_thumb),
			"on_previous_image1_activate" : (self.__on_previous_thumb),
			"on_entry1_activate" : (self.__onFolderEntryActivate),
			"on_about_activate" : (self.__onAboutActivate)
		}
		glade_xml.signal_autoconnect(handlers)
		self.window.show()

	#
	# Private member functions
	#
	def __getDirTreeModel(self):
		return self.dirTreeModel.getModel()
	
	def __getFolder(self, rowIter):
		return self.__getDirTreeModel().get_value(rowIter, 1)
	
	def __setThumbFolder(self, folder):
		self.thumbGrid.setDirectory(folder)

	def __getImageTargetSize(self):
		return self.imageContainer.get_allocation()

	def __getPreviewTargetSize(self):
		return self.previewNotebook.get_allocation()

	def __getImageAtCursorLocation(self, x, y):
		self.thumbGrid.getImageForIconAt(x, y)

	def __getSelectedImage(self):
		return self.thumbGrid.getSelectedImage()

	def __isAnyIconSelected(self):
		return self.thumbGrid.getSelectedIcon() != ThumbGrid.NO_SELECTION		

	def __setRootFolder(self, folder):
		self.rootFolder = folder
		self.dirTreeModel.setDirectory(folder)
	
	def __showBigImage(self, modelImage):
		print "showing image", modelImage
		if modelImage == None: return
		image = self.imageContainer.get_child().get_child()
		modelImage.show(image, self.__getImageTargetSize())
		gc.collect()
	
	def __showPreviewImage(self, modelImage):
		if modelImage == None: return
		image = self.previewNotebook.get_children()[1]
		modelImage.show(image, self.__getPreviewTargetSize())
		
	def __showEXIF(self, modelImage, textViewForExif):
		textBuffer = gtk.TextBuffer()
		textBuffer.set_text(modelImage.getEXIF())
		textViewForExif.set_buffer(textBuffer)

	def __restoreWindowSize(self, config, window):
		if config["window-height"] != None and config["window-width"] != None:
			print "restoring size ", config["window-height"], config["window-width"]
			window.resize(int(config["window-width"]), int(config["window-height"]))
	
	def __saveWindowSize(self, config, window):
		size = window.get_allocation()
		print "saving window size", size.width, size.height
		config["window-width"] = size.width
		config["window-height"] = size.height

	#
	# Event handlers
	#
	def __onFolderEntryActivate(self, entry):
		print entry.get_text()
		if isDirectory(entry.get_text()):
			self.__setRootFolder(entry.get_text())
			self.thumbGrid.setDirectory(entry.get_text())
			
	def __onThumbnailClicked(self, widget, event):
		if event.type == gtk.gdk._2BUTTON_PRESS:
			# double click occurred, show clicked image
			self.imageNotebook.set_current_page(1)
			#self.__showBigImage(self.__getImageAtCursorLocation(event.x, event.y))
			self.__showBigImage(self.__getSelectedImage())
		
	def __onSelectFolder(self, treeView, startEditing):
		model, row = treeView.get_selection().get_selected()
		folder = model.get_value(row, 1)
		if folder:
			self.__setThumbFolder(folder)
		
	def __onRowExpanded(self, treeView, row, path):
		print "onExpand"
		if self.__getFolder(row):
			print "loading subfolders"
			self.dirTreeModel.setSubDirectory(row, self.__getFolder(row))

	def __onFolderActivate(self, treeView, path, column):
		self.__onSelectFolder(treeView, None)
 		
	def __on_thumbnailsmenuitem_activate(self, menuItem):
		self.imageNotebook.set_current_page(0)
 		
	def __on_imagemenuitem_activate(self, menuItem):
		if self.__isAnyIconSelected():
			self.__showBigImage(self.__getSelectedImage())
		self.imageNotebook.set_current_page(1)

	def __onSelectIcon(self, iconList, icon, event, textViewForExif):
		self.thumbGrid.onSelectIcon(iconList, icon, event)
		
		if self.__getSelectedImage() == None: return

		if previewNotebook.get_current_page() == 0:
			self.__showEXIF(self.__getSelectedImage(), textViewForExif)	
		if self.previewNotebook.get_current_page() == 1:
			print "showing preview"
			self.__showPreviewImage(self.__getSelectedImage())
			self.previewNotebook.set_current_page(1)

	def __on_key(self, widget, event, image):
		if (event.keyval == Page_Up):
			self.thumbGrid.selectPrevious()
		if (event.keyval == Page_Down):
			self.thumbGrid.selectNext()
		self.__showBigImage(self.__getSelectedImage())
		
	def __on_next_thumb(self, menuItem):
		self.thumbGrid.selectNext()
		self.__showBigImage(self.__getSelectedImage())
		
	def __on_previous_thumb(self, menuItem):
		self.thumbGrid.selectPrevious()
		self.__showBigImage(self.__getSelectedImage())
	
	def __onAboutActivate(self, obj):
	    """Displays the about dialog"""
	    about = self.xml.get_widget("about")
	    about.show ()

	def __onDeleteEvent(self, widget, event, data=None):
		print "delete event occurred"
		return gtk.FALSE

	def __onDestroy(self, widget, data=None):
		self.__saveWindowSize(self.config, self.window)
 		gtk.main_quit()
	    