#	  This file is part of Dim, a Digital Image Management program.
#	  Copyright 2003, Anssi Piirainen
#    Dim/DirTreeModel.py - Directory model for gtk.TreeStore
#
#    Dim is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Dim is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Dim; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
import gtk
import gobject
import os

from dimlib.Library import isDirectory

class DirTreeModel:
	
	def __init__(self):
		self.model = gtk.TreeStore(gobject.TYPE_STRING, gobject.TYPE_PYOBJECT)
	
	def getModel(self):
		return self.model
	
	def setDirectory(self, directory):		
		self.model.clear()
		self.__walkDirectory(directory)
	
	def __walkDirectory(self, directory, parent=None, depth=1):
		print "DirTreeModel.__walkDirectory, directory = ", directory
		
		try:
			dirlist = os.listdir(directory)
		except OSError:
			return [ ]
		
		dirlist =  [f for f in dirlist if f[0] != "."]
		for f in dirlist:
			fullPath = os.path.join(directory, f)
			if isDirectory(fullPath):
				newRow = self.model.append(parent, None)
				self.model.set_value(newRow, 0, f)
				self.model.set_value(newRow, 1, fullPath)
				
				## recursively add subdirectories to depth 2
				#if (depth < 2):
				#	self.__walkDirectory(fullPath, newRow, depth+1)
				#else:
				newRow = self.model.append(newRow, None)
				self.model.set_value(newRow, 0, "loading...")
				self.model.set_value(newRow, 1, "loading...")
					
	def setSubDirectory(self, parentRow, subDir):
		child = self.model.iter_children(parentRow)
		if self.model.get_value(child, 0) == "loading...":
			self.model.remove(child)
			self.__walkDirectory(subDir, parentRow)



