#	  This file is part of Dim, a Digital Image Management program.
#	  Copyright 2003, Anssi Piirainen
#    Dim/ThumbGrid.py - gtkI.IconList that shows thumbnails of images
#
#    Dim is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Dim is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Dim; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
import os
import thread
import gtk
import gnome.ui

from dimlib import *

NO_SELECTION = -1

class ThumbGrid(gnome.ui.IconList):
	def __init__(self, imageIndex):
		gnome.ui.IconList.__init__(self, 150, gtk.Adjustment(), False)
		
		self.__imageIndex = imageIndex

		self.__imageIndex.setImageListener(self)
		self.images = { }
		self.selectedIcon = NO_SELECTION

	def setDirectory(self, directory):
		self.clear()
		images = self.__imageIndex.getFolderImages(directory)
		if images != None:
			map(lambda image: (self.onImageCreated(image)), images)

		self.lock = thread.allocate_lock()
		self.lock.acquire()
		thread.start_new_thread(self.__getImages, (directory, ))
		while (self.lock.locked()):
			while (gtk.events_pending()): gtk.mainiteration(False)
	
	def __getImages(self, directory):
		self.__imageIndex.createFolderImages(directory)
	
	def onImageCreated(self, image, object=None):
		iconIndex = self.append(image.getThumb(), image.getFileBasename())
		self.images[iconIndex] = image
		self.queue_draw()
	
	def onImageFolderLoaded(self, list, object):
		print "received event for folder loaded", list
		self.lock.release()
		thread.exit_thread()

	def onSelectIcon(self, iconList, icon, event):
		self.selectedIcon = icon

	def getSelectedIcon(self):
		return self.selectedIcon

	def getImageForIcon(self, icon):
		return self.images[icon]

	def getSelectedImage(self):
		return self.getSelectedIcon() != NO_SELECTION and self.images[self.getSelectedIcon()] or None
	
	def selectPrevious(self):
		if self.getSelectedIcon() == NO_SELECTION: return
		if self.getSelectedIcon() - 1 < 0: return
		self.select_icon(self.getSelectedIcon() - 1)
	
	def selectNext(self):
		if self.getSelectedIcon() == NO_SELECTION: return
		if (self.getSelectedIcon() + 1) > self.get_num_icons(): return
		self.select_icon(self.getSelectedIcon() + 1)

	def getImageForIconAt(self, x, y):
		return self.getImageForIcon(self.get_icon_at(x, y))